#!/bin/bash
source_dir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
INI_FILE_NAME=output_config.ini
outputdir=
normalize=false
TMP_IBDIAGNET_OUTPUT_DIR=$PWD/tmp_ibdiagnet_dir
ibdiagnet_output_dir=$TMP_IBDIAGNET_OUTPUT_DIR
IBDIAGNET_ARGS="-r --routers --get_cable_info --enable_output all -o $TMP_IBDIAGNET_OUTPUT_DIR "
IBDIAGNET_EXE="${IBDIAGNET_EXE:-`which ibdiagnet`}"
GEN_OUTPUT_SCRIPT=$source_dir/generate_output.py
OUTPUT_INI=$source_dir/$INI_FILE_NAME

usage() {
cat <<HELP_USAGE

A wrapper script for output files processing. Generates heatmaps of bandwidth, latency and normalized latency,
puts all heatmaps in a zip file and creates a tgz of JSON files in output directory.
NOTE - set IBDIAGNET_EXE for location of ibdiagnet executable, otherwise will use default path.

$0 -o|--output <dir> [-l|--normalize]

  Parameters:
	-o|--outputdir <dir>        Path to clusterkit output directory
	-l|--normalize              Normalize latency.json. Requires either an ibdiagnet data_dir field (see INI file),
	                            or an execution of ibdiagnet within the script, which requires sudo permission


  Configure execution using INI file $OUTPUT_INI :

  [DEFAULT] section:
      verbose            Boolean, set to 1/0 for more/less information while running, respectively


  [ibdiagnet] section:
      ** Relevant when running script with --normalize
      hca_device	 HCA to use when running ibdiagnet on the fabric
      data_dir  	 Directory with the following ibdiagnet output files:
                	      [ibdiagnet2.db_csv, ibdiagnet2.fdbs, ibdiagnet2.cables, ibdiagnet2.nodes_info]

  [normalize_latency] section:
      per_hca            Boolean, 1 if clusterkit execution was per HCA and not per node, otherwise 0
      config_file	 CSV file of devices and optical cables. Each line can be either a device ID with
        	         worst case latency in ns or an optical cable part number.
	                 For example see latency_calc_config.csv

  [render_heatmap] section:
			 Both flags has a prefix of 'bandwidth'/'latency', where 'latency' thresholds
			 are used for normalized latency only.
      thresholds_file    File with threshold values (1 per line) for heatmap color scale.
                         See 'thresholds' examples and thresholds.txt. Is ignored if 'thresholds' is set.
      thresholds         Comma separated list of threshold values for heatmap color scale.
			 Can contain 1/2/3/6 values to set 2/3/4/7 color scale bar, respectively.
                         The values can contain the following variables:
                           ['average','Average','avg','mean','std','STD','sigma'].
                         Examples (in INI file, in render_heatmap section):
                              latency_thresholds=0.4,0.5,0.6,0.7,0.8,0.9
                              # 7 colors scale for latency in usec

                              bandwidth_thresholds=avg-std,avg+std,avg+2*std
                              # 4 colors scale with jumps of standard deviation around the mean for bandwidth heatmap

                              latency_thresholds=mean-0.1,mean+0.1
                              # 3 colors scale for latencies under mean-0.1 usec, between mean-0.1 to mean+0.1
                              # and above mean+0.1

                              bandwidth_thresholds=mean
                              # 2 colors scale for below and above the mean of bandwidth
HELP_USAGE
}

get_ib_devices() {
        devices=$( ibstat -l )
        echo "${devices}"
}

do_start() {
	echo "Running virtual environment..."
	$python -m venv venv
	source venv/bin/activate
	pip install --upgrade pip > /dev/null
	pip install -r $source_dir/requirements.txt > /dev/null
}

clean_venv() {
	echo "Cleaning virtual environment..."
	deactivate
	rm -rf venv
}

do_end() {
	clean_venv
	rm -rf $TMP_IBDIAGNET_OUTPUT_DIR
}

render() {
	json_file=$1
	config_file=$2
	gaussian=$3
	if [[ $json_file && $config_file ]]; then
		echo "Rendering  ${json_file}"
		python $GEN_OUTPUT_SCRIPT -r -f $config_file -j $json_file $gaussian
	fi
}

find_loop_render() {
	token=$1
	gaussian=$2
	find $outputdir -name $token | while read line; do
		file_name=$( realpath $line )
		render $file_name $OUTPUT_INI $gaussian
	done
}

render_output_files() {
	find_loop_render '*latency*.json' -g
	find_loop_render '*bandwidth*.json'
}

check_ibd_dir() {
	local code=0
	data_dir="${1}"
	if [[ ! -d "${data_dir}" ]]; then
		echo "ERROR: ${data_dir} directory not exists/ is not a directory"
		return 1
	fi

	files="ibdiagnet2.db_csv ibdiagnet2.fdbs ibdiagnet2.cables ibdiagnet2.nodes_info"
	for f in $files;do
		if [[ ! -f "${data_dir}/$f" ]]; then
			echo "ERROR: missing file $f in ${data_dir}"
			code=1
		fi
	done
	return $code
}

run_ibdiagnet_all_devices() {
	devices=$( get_ib_devices )
	cables_file="${TMP_IBDIAGNET_OUTPUT_DIR}/ibdiagnet2.cables"
	for d in $devices; do
		sudo $IBDIAGNET_EXE $IBDIAGNET_ARGS -i $d >/dev/null
		if [[ -f "$cables_file" ]]; then
			return 0
		fi
		rm -rf $TMP_IBDIAGNET_OUTPUT_DIR
	done
	return 1
}

run_ibdiagnet() {
	if [ -z "$IBDIAGNET_EXE" ]; then
		echo "IBDIAGNET_EXE was not set and no location of ibdiagnet was found."
		return
	fi

	data_dir=$( awk -F "=" '/data_dir/ {print $2}' $OUTPUT_INI )
	if [[ -n "${data_dir}" ]]; then
		check_ibd_dir "${data_dir}"
		local ret=$?
		if [[ $ret -ne 0 ]]; then
			return 1
		fi
		ibdiagnet_output_dir="${data_dir}"
		return 0
	fi

	echo -n "Checking sudo permission for ibdiagnet... "
	out=$( timeout --kill-after=1 1 sudo whoami )
	if [ "$out" = root ];then
		echo "yes"
	else
		echo "no"
		return 1
	fi

	hca_arg=$( awk -F "=" '/hca_device/ {print $2}' $OUTPUT_INI )
	if [[ $hca_arg ]]; then
		sudo $IBDIAGNET_EXE $IBDIAGNET_ARGS -i $hca_arg >/dev/null
	else
		run_ibdiagnet_all_devices
	fi
	return $?
}

normalize_latency_and_render() {
	run_ibdiagnet
	local ret=$?
	if [[ $ret -ne 0 ]];then
		echo "Cannot normalize latency"
		return
	fi

	latencyfile=$( realpath $( find $outputdir -maxdepth 1 -name "latency.json" ) )
	echo "Normalizing ${latencyfile}"
	python $GEN_OUTPUT_SCRIPT -n -f $OUTPUT_INI -j $latencyfile -d $ibdiagnet_output_dir
	exit_status=$?
	if [ $exit_status -ne 0 ]; then
		echo "An error occurred while normalizing ${latencyfile}, cannot render"
		return
	fi

	normalized_file=$( realpath $( find $outputdir -name "*_normalized.json" ) )
	render $normalized_file $OUTPUT_INI
}

compress_output_files() {
	# zip all heatmap images
	cd $outputdir
	pngs=$( find -name "*.png" )
	if [ -z "$pngs" ]; then
		echo "Did not find any heatmaps in $outputdir"
	else
		echo Creating zip file..
		zip processed_results.zip $pngs
	fi

	# tgz all json files
	jsons=$( find -name "*.json" )
	if [ -z "$jsons" ]; then
		echo "Did not find any JSON file in $outputdir"
	else
		echo Creating tgz file...
		name=$( basename $outputdir )
		tar -czvf "${name}.tgz" $jsons
	fi
	cd - &> /dev/null
}

while [[ $# -gt 0 ]]
do
	key="${1}"
	case ${key} in
		-o|--outputdir)
			outputdir="${2}"
			shift # past argument
			;;
		-l|--normalize)
			normalize=true
			;;
		-h|--help)
			usage
			exit 0
			;;
		*)    # unknown option
			echo Error: unknown option $key
			usage
			exit 1
			;;
	esac
    shift
done

if [ -z $outputdir ]; then
	usage
	exit 1
fi
if [ ! -d $outputdir ];then
  echo "Error: ${outputdir} directory not exists/ is not a directory"
  exit 1
fi

python="python"
version=$( python -V 2>&1 )
if [[ "$version" == *" 2."* ]];then
	python+="3"
fi

do_start
render_output_files
if [[ "$normalize" = true ]]; then
  normalize_latency_and_render
fi
compress_output_files
do_end
